import { repeat } from './tool';
// 最简单的Decorator，即在现有文字的基础上加上前缀、后缀即可
var SIMPLE_DECORATOR = {
  bold: ['**', '**'],
  italic: ['*', '*'],
  underline: ['++', '++'],
  strikethrough: ['~~', '~~'],
  quote: ['\n> ', '\n'],
  inlinecode: ['`', '`'],
  code: ['\n```\n', '\n```\n'],
  mainColors0: ['<font color="#000">', '</font>'],
  mainColors1: ['<font color="#333">', '</font>'],
  mainColors2: ['<font color="#444">', '</font>'],
  mainColors3: ['<font color="#555">', '</font>'],
  mainColors4: ['<font color="#666">', '</font>'],
  mainColors5: ['<font color="#777">', '</font>'],
  mainColors6: ['<font color="#888">', '</font>'],
  mainColors7: ['<font color="#999">', '</font>'],
  mainColors8: ['<font color="#aaa">', '</font>'],
  mainColors9: ['<font color="#bbb">', '</font>'],
  mainColors10: ['<font color="#ccc">', '</font>'],
  mainColors11: ['<font color="#ddd">', '</font>'],
  mainColors12: ['<font color="#eee">', '</font>'],
  mainColors13: ['<font color="#fff">', '</font>'],
  colors0: ['<font color="#fec6c2">', '</font>'],
  colors1: ['<font color="#fee5c3">', '</font>'],
  colors2: ['<font color="#fefcc3">', '</font>'],
  colors3: ['<font color="#baf6c4">', '</font>'],
  colors4: ['<font color="#c3ebfe">', '</font>'],
  colors5: ['<font color="#c3cbfe">', '</font>'],
  colors6: ['<font color="#e1caff">', '</font>'],
  colors7: ['<font color="#fc8e88">', '</font>'],
  colors8: ['<font color="#fccc88">', '</font>'],
  colors9: ['<font color="#fcf888">', '</font>'],
  colors10: ['<font color="#76ec8a">', '</font>'],
  colors11: ['<font color="#88d8fc">', '</font>'],
  colors12: ['<font color="#97a4fb">', '</font>'],
  colors13: ['<font color="#c098f4">', '</font>'],
  colors14: ['<font color="#ff6666">', '</font>'],
  colors15: ['<font color="#ffb151">', '</font>'],
  colors16: ['<font color="#fada3a">', '</font>'],
  colors17: ['<font color="#18c937">', '</font>'],
  colors18: ['<font color="#3aaafa">', '</font>'],
  colors19: ['<font color="#6373e2">', '</font>'],
  colors20: ['<font color="#a669f7">', '</font>'],
  colors21: ['<font color="#f63030">', '</font>'],
  colors22: ['<font color="#f88933">', '</font>'],
  colors23: ['<font color="#deb12a">', '</font>']
}; // 插入H1-H6

for (var i = 1; i <= 6; i++) {
  SIMPLE_DECORATOR["h" + i] = ["\n" + repeat('#', i) + " ", '\n'];
}

function decorateTableText(option) {
  var _option$row = option.row,
      row = _option$row === void 0 ? 2 : _option$row,
      _option$col = option.col,
      col = _option$col === void 0 ? 2 : _option$col;
  var rowHeader = ['|'];
  var rowData = ['|'];
  var rowDivision = ['|'];
  var colStr = '';

  for (var _i = 1; _i <= col; _i++) {
    rowHeader.push(' Head |');
    rowDivision.push(' --- |');
    rowData.push(' Data |');
  }

  for (var j = 1; j <= row; j++) {
    colStr += '\n' + rowData.join('');
  }

  return rowHeader.join('') + "\n" + rowDivision.join('') + colStr;
}

function decorateList(type, target) {
  var text = target;

  if (text.substr(0, 1) !== '\n') {
    text = '\n' + text;
  }

  if (type === 'unordered') {
    return text.length > 1 ? text.replace(/\n/g, '\n* ').trim() : '* ';
  } else {
    var count = 1;

    if (text.length > 1) {
      return text.replace(/\n/g, function () {
        return "\n" + count++ + ". ";
      }).trim();
    } else {
      return '1. ';
    }
  }
}

function createTextDecorated(text, newBlock) {
  return {
    text: text,
    newBlock: newBlock,
    selection: {
      start: text.length,
      end: text.length
    }
  };
}
/**
 * 获取装饰后的Markdown文本
 * @param target 原文字
 * @param type 装饰类型
 * @param option 附加参数
 * @returns {Decorated}
 */


function getDecorated(target, type, option) {
  if (typeof SIMPLE_DECORATOR[type] !== 'undefined') {
    return {
      text: "" + SIMPLE_DECORATOR[type][0] + target + SIMPLE_DECORATOR[type][1],
      selection: {
        start: SIMPLE_DECORATOR[type][0].length,
        end: SIMPLE_DECORATOR[type][0].length + target.length
      }
    };
  }

  switch (type) {
    case 'tab':
      var inputValue = option.tabMapValue === 1 ? '\t' : ' '.repeat(option.tabMapValue);
      var newSelectedText = inputValue + target.replace(/\n/g, "\n" + inputValue);
      var lineBreakCount = target.includes('\n') ? target.match(/\n/g).length : 0;
      return {
        text: newSelectedText,
        selection: {
          start: option.tabMapValue,
          end: option.tabMapValue * (lineBreakCount + 1) + target.length
        }
      };

    case 'unordered':
      return createTextDecorated(decorateList('unordered', target), true);

    case 'order':
      return createTextDecorated(decorateList('order', target), true);

    case 'hr':
      return createTextDecorated('---', true);

    case 'table':
      return {
        text: decorateTableText(option),
        newBlock: true
      };

    case 'image':
      return {
        text: "![" + (target || option.target) + "](" + (option.imageUrl || '') + ")",
        selection: {
          start: 2,
          end: target.length + 2
        }
      };

    case 'link':
      return {
        text: "[" + target + "](" + (option.linkUrl || '') + ")",
        selection: {
          start: 1,
          end: target.length + 1
        }
      };
  }

  return {
    text: target,
    selection: {
      start: 0,
      end: target.length
    }
  };
}

export default getDecorated;